// ===========================================================================
//	File:						CMPDPanelSelectControl.h
// Version:					1.0 - Feb 1, 1996
//	Author:					Mike Shields (mshields@inconnect.com)
//							
//	Copyright 1996 Mike Shields. All rights reserved.
// ===========================================================================
//
// Abstract class which describes the methods a PanelSelect control for the
// Multipane dialog architecture must implement.

#pragma once

#include "MPDTypes.h"
#include "UMPDialogHandler.h"

class CMPDPanelSelectControl 
{
public:
	enum				{ index_Bad		= 0,
						  index_First	= 1,
						  index_Last	= 0x7FFF };
						  
									CMPDPanelSelectControl()
										: mMyMPDHandler(nil) { };
	virtual						~CMPDPanelSelectControl() { };
	
	// Subclasses must overide this method to insert a new panel ID into its 
	// list of categories to display
	virtual void				InsertPanelIDs(MPDPtr inPanelIDData) = 0;

	// Subclasses must overide this method to select the current panel ID
	// which is being displayed.
	virtual void				SelectPanelID(PanelIDIndexT inPanelID) = 0;
	
	virtual void 				ReportPanelIDChanged(void)
									{
										if ( mMyMPDHandler)
											mMyMPDHandler->HandlePanelChange(GetCurrentPanelID());
									};

	virtual void				ConnectToHandler(StMPDialogHandler* inHandler)
									{
										mMyMPDHandler = inHandler;
									};
	
protected:
	virtual PanelIDIndexT	GetCurrentPanelID(void) const = 0;
	
	StMPDialogHandler*		mMyMPDHandler;
};

